#
#

available_modules = """Available:
    phlop.run.test_cases -h
    phlop.run.stats_man -h
    phlop.run.mpirun_stats_man -h
    phlop.run.perf -h
    phlop.run.valgrind -h"""

print(available_modules)
